/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drparker@freenet.co.uk                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "app.h"
#include "impl.h"
#include "screen.h"
#include "image.h"
#include "font.h"
#include "input.h"
#include <iostream>

using namespace std;

// Main application class

App::App(Impl* p) : impl(p)
{
    // Initialize SDL's subsystems - in this case, only video.
    if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0 ) 
    {
	cerr << "Unable to init SDL: " << SDL_GetError() << endl;
	exit(1);
    }

    // Register SDL_Quit to be called at exit; makes sure things are
    // cleaned up when we quit.
    atexit(SDL_Quit);
}

void App::Main(void)
{
    // Attempt to create a 320x240 window with 16bit pixels.
    Screen screen(320, 240);

    // Hide cursor
    SDL_ShowCursor (SDL_DISABLE);

    // Create input processor
    Input input;

    if (impl != NULL)
    {
	// Initialise app specific stuff
	impl->Initialise(screen);
    }

    // Main loop
    while(true)
    {
	screen.Lock();
	if (impl != NULL)
	{
	    // Do app rendering
	    impl->Render(screen);
	}
	screen.Unlock();
	screen.Update();

	if (impl != NULL)
	{
	    // Do other stuff
	    impl->NonRender(input);
	}

	// Handle SDL events
	SDL_Event event;
	while (SDL_PollEvent(&event))
	{
	    switch (event.type)
	    {

		// Handle keys 
		// a - a
		// b - b
		// x - x
		// y - y
		// l - l
		// r - r
		// + - vol+
		// - - vol-
		// Return - select
		// Start  - space
		// Cursor keys - joystick

	    case SDL_KEYDOWN:
	    {
		Input::ButtonType b = input.Translate(event.key.keysym.sym);
		if (b != Input::NUMBER_OF_BUTTONS)
		{
		    input.Down(b);
		    if (impl) impl->ButtonChange(input,b);
		}
	    }
	    break;

	    case SDL_KEYUP:
	    {
		// If escape is pressed, return (and thus, quit)
		if (event.key.keysym.sym == SDLK_ESCAPE)
		    return;

		Input::ButtonType b = input.Translate(event.key.keysym.sym);
		if (b != Input::NUMBER_OF_BUTTONS)
		{
		    input.Up(b);
		    if (impl) impl->ButtonChange(input, b);
		}
	    }
	    break;
	    
	    // Handle GP2X button events

	    case SDL_JOYBUTTONUP:
	    {
		Input::ButtonType b = static_cast<Input::ButtonType>(event.jbutton.button);
		input.Up(b);
		if (impl) impl->ButtonChange(input, b);
	    }
	    break;
	    
	    case SDL_JOYBUTTONDOWN:
	    {
		Input::ButtonType b = static_cast<Input::ButtonType>(event.jbutton.button);
		input.Down(b);
		if (impl) impl->ButtonChange(input, b);
	    }
	    break;

	    case SDL_QUIT:
		return;
	    }
	}
    }
}
