/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drparker@freenet.co.uk                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "input.h"
#include <iostream>
#include <SDL.h>

using namespace std;

// Input handler

Input::Input(void)
{
    InitJoy();
}

// Initialise GP2X joystick
void Input::InitJoy(void)
{
    joy = NULL;

    if (SDL_InitSubSystem (SDL_INIT_JOYSTICK) == -1)
    {
        cerr << "Unable to initialise joystick: " << SDL_GetError() << endl;
	exit(1);
    }
    
    if ( SDL_NumJoysticks() > 0)
    {
	joy = SDL_JoystickOpen(0);
    }


    if (joy)
    {
	SDL_JoystickEventState (SDL_ENABLE);	
    }
}

// Button pressed
void Input::Down(ButtonType b)
{
    buttons[b].Down();
}

// Button released
void Input::Up(ButtonType b)
{
    buttons[b].Up();
}

// Returns true if button is pressed
bool Input::IsPressed(ButtonType b) const
{
    return buttons[b].IsPressed();
} 

// Translate key to buttons
Input::ButtonType Input::Translate(int key)
{
    ButtonType res = NUMBER_OF_BUTTONS;
    switch (key)
    {
    case SDLK_a:
	res = A;
	break;

    case SDLK_b:
	res = B;
	break;

    case SDLK_x:
	res = X;
	break;

    case SDLK_y:
	res = Y;
	break;

    case SDLK_SPACE:
	res = START;
	break;

    case SDLK_RETURN:
	res = SELECT;
	break;

    case SDLK_l:
	res = L;
	break;

    case SDLK_r:
	res = R;
	break;

    case SDLK_EQUALS:
	res = VOLUP;
	break;

    case SDLK_MINUS:
	res = VOLDOWN;
	break;

    case SDLK_LEFT:
	res = LEFT;
	break;

    case SDLK_RIGHT:
	res = RIGHT;
	break;

    case SDLK_UP:
	res = UP;
	break;

    case SDLK_DOWN:
	res = DOWN;
	break;
    }

    return res;
}

// Returns true if button repeats
bool Input::Repeat(ButtonType b) const
{
    return (buttons[b].Repeat() && buttons[b].IsPressed());
}
