/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drparker@freenet.co.uk                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INPUT_H
#define INPUT_H

#include <SDL.h>
#include "button.h"

class Input
{
public:

    // GP2X button values
    enum ButtonType
    {
	UP,
	UPLEFT,
	LEFT,
	DOWNLEFT,
	DOWN,
	DOWNRIGHT,
	RIGHT,
	UPRIGHT,
	START,
	SELECT,
	L,
	R,
	A,
	B,
	Y,
	X,
	VOLUP,
	VOLDOWN,
	CLICK,
	NUMBER_OF_BUTTONS
    };

    // Constructor
    Input(void);

    // Button pressed
    void Down(ButtonType b);

    // Button released
    void Up(ButtonType b);

    // Return true if button is pressed
    bool IsPressed(ButtonType b) const;

    // Translate key to button
    ButtonType Translate(int key);

    // Returns true if button repeats
    bool Repeat(ButtonType b) const;

private:
    // SDL joystick handle
    SDL_Joystick* joy;

    // Initise joystick
    void InitJoy(void);

    // Button states
    Button buttons[NUMBER_OF_BUTTONS];
};

#endif // INPUT_H
