/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drparker@freenet.co.uk                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "screen.h"
#include "colour.h"
#include <iostream>

using namespace std;

// Construct 16 bit colour screen of given size
Screen::Screen (int w, int h) : width(w), height(h)
{
    screen = SDL_SetVideoMode(w, h, 16, SDL_SWSURFACE|SDL_HWPALETTE);   
}

// Draw a rectangle
void Screen::Rect (int x, int y, int w, int h, const Colour& c)
{
    SDL_Rect r;
    r.x = x;
    r.y = y;
    r.w = w;
    r.h = h;
    SDL_FillRect (screen, &r, c.Get());   
}

// Lock screen
void Screen::Lock(void)
{
    if (SDL_MUSTLOCK(screen)) 
	if (SDL_LockSurface(screen) < 0) 
	    return;    
}

// Unlock screen
void Screen::Unlock(void)
{
    if (SDL_MUSTLOCK(screen)) 
	SDL_UnlockSurface(screen);  
}

// Update whole screen
void Screen::Update(void)
{
    SDL_UpdateRect(screen, 0, 0, width, height); 
}

// Put pixel of colour c at x,y
// If colour is NONE - no pixel is ploted
void Screen::PutPixel (int x, int y, const Colour& c)
{
    if (x >=0 && x < screen->w && y >=0 && y < screen->h && 
	c != Colour::NONE)
    {
	unsigned short* dst = static_cast<unsigned short*>(screen->pixels);
	dst[y * screen->pitch/sizeof(unsigned short) + x] = c.Get();
    }    
}

// Get screen SDL surface
SDL_Surface* Screen::get_screen(void) const
{
    return screen;
}

// Get width
int Screen::get_width(void) const
{
    return width;
}

// Get height
int Screen::get_height(void) const 
{
    return height;
}

// Get format
SDL_PixelFormat* Screen::Format(void) const
{
    return screen->format;
}
