/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drparker@freenet.co.uk                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define MAXWORD 30
#define MAXLINE 120

#define TRUE    1
#define FALSE   0

#define HIVAL   0

typedef struct word_rec
{
        int proper;
        int value;
        char word[MAXWORD];
        struct word_rec *next;
} ENTRY, *ENTRY_PTR;

typedef struct head_rec
{
        struct word_rec *letter[26];
} HEADER, *HEAD_PTR;

char dict[MAXLINE] = "./.dictionary/DICTIONARY";
char sdict[MAXLINE] = "./.dictionary/SUPP";

char vchars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

HEADER head;

ENTRY_PTR add_entry(ENTRY_PTR list, ENTRY_PTR element);
ENTRY_PTR get_rec();
void add_to_dictionary(char* dictionary);
void anag(char* letters, char* number);
int check_mask(char* mask);
void init_from_dictionary(char* dictionary);
int in_word(char* letters, char* word, char* rem);
void list_entry(ENTRY_PTR ptr, char* mask);
void list_word(char* mask);
void list_words_in(ENTRY_PTR ptr, char* letters,char* mask);
int match (char* word, char* mask);
void mix(HEADER list, char* letters, char* line, char* rem,
	 int t, int l, int m);
char *subnstr(char* str, char* sub, int n);
void words_in_seq(char* line, char* mask);

/* MAIN */

int main(int argc, char** argv)
{
        int i, opt;
        char s1[MAXLINE];
        char s2[MAXLINE];

        *s1 = '\0';
        *s2 = '\0';

        if (argc < 3)
        {
                fprintf(stderr,"Usage : %s -a <str> [<no words>] | ",argv[0]);
                fprintf(stderr,"-l [<mask>] | -s <str> [<mask>]\n");
                return 1;
        }
        nice(10);

        for (i=0; i<26; i++)
        {
                head.letter[i] = NULL;
        }

        init_from_dictionary(dict);
        add_to_dictionary(sdict);

        if (argc > 2)
        {
                strcpy(s1,argv[2]);
        }
        if (argc > 3)
        {
                strcpy(s2,argv[3]);
        }
        opt = argv[1][1];
        switch (opt)
        {
                case 'l':
                        list_word(s1);
                        break;

                case 'a':
                        anag(s1,s2);
                        break;

                case 's':
                        words_in_seq(s1,s2);
                        break;
        }
	return 0;
}

void init_from_dictionary(char* dictionary)
{
        int i;
        char line[MAXLINE], *s;
        FILE *infile;
        ENTRY_PTR ptr;
        HEADER cptr;

        for (i=0; i<26; i++)
        {
                cptr.letter[i] = head.letter[i];
        }

        if ((infile = fopen(dictionary,"r")) == NULL)
        {
                return;
        }

        while (fgets(line,MAXLINE,infile) != NULL)
        {
                if(strchr(vchars,*line) != NULL)
                {
                        ptr = get_rec();
                        if (isupper(*line))
                        {
                                *line = tolower(*line);
                                ptr->proper = TRUE;
                        }
                        line[strlen(line)-1] = '\0';
                        if ((s=strchr(line,' ')) != NULL)
                        {
                                ptr->value = atoi(s+1);
                                *s = '\0';
                        }
                        strcpy(ptr->word,line);

                        if (cptr.letter[*line-'a'] == NULL)
                        {
                                head.letter[*line-'a'] = ptr;
                                cptr.letter[*line-'a'] = ptr;
                        }
                        else
                        {
                                cptr.letter[*line-'a']->next = ptr;
                                cptr.letter[*line-'a'] = ptr;
                        }
                }       
        }
}

void add_to_dictionary(char* dictionary)
{
        int i;
        char line[MAXLINE], *s;
        FILE *infile;
        ENTRY_PTR ptr;

        if ((infile = fopen(dictionary,"r")) == NULL)
        {
                return;
        }

        while (fgets(line,MAXLINE,infile) != NULL)
        {
                if(strchr(vchars,*line) != NULL)
                {
                        ptr = get_rec();
                        if (isupper(*line))
                        {
                                *line = tolower(*line);
                                ptr->proper = TRUE;
                        }
                        line[strlen(line)-1] = '\0';
                        if ((s=strchr(line,' ')) != NULL)
                        {
                                ptr->value = atoi(s+1);
                                *s = '\0';
                        }
                        strcpy(ptr->word,line);

                        head.letter[*line-'a'] = 
                             add_entry(head.letter[*line-'a'], ptr);
                }
        }
}


ENTRY_PTR get_rec()
{
        ENTRY_PTR ptr;

        ptr = (ENTRY_PTR) malloc(sizeof(ENTRY));
        ptr->proper = FALSE;
        ptr->value = 0;
        ptr->proper = FALSE;
        ptr->word[0] = '\0';
        ptr->next = NULL;

        return ptr;
}

void list_word(char* mask)
{
        int i,error;

        error = check_mask(mask);
        if (*mask == '\0')
        {
                strcpy(mask,"*");
        }
        if (!error)
        {
                if (*mask >= 'a' && *mask <= 'z')
                {
                        list_entry(head.letter[*mask-'a'], mask);
                }
                else
                {
                        for (i=0; i<26; i++)
                        {
                                list_entry(head.letter[i], mask);
                        }
                }
        }
}

void list_entry(ENTRY_PTR ptr, char* mask)
{
        char tword[MAXWORD];

        if (ptr == NULL)
        {
                return;
        }

        if (match(ptr->word,mask))
        {
                strcpy(tword,ptr->word);
                if (ptr->proper)
                {
                        *tword = toupper(*tword);
                }
                if (ptr->value > HIVAL)
                {
                        printf("<%s>\n",tword);
                }
                else
                {
                        printf("%s\n",tword);
                }
                fflush(stdout);
        }
        list_entry(ptr->next,mask);
}

void words_in_seq(char* line, char* mask)
{
        int i,error;

        error = FALSE;
        if (*line == '\0')
        {
                return;
        }
        for (i=0; line[i] != '\0' && !error; i++)
        {
                line[i] = tolower(line[i]);
                if (strchr(vchars,line[i]) == NULL)
                {
                        printf("Invalid character (%c)\n",line[i]);
                        error = TRUE;
                }
        }

        if (!error)
        {
                error = check_mask(mask);
        }

        if (*mask == '\0')
        {
                strcpy(mask,"*");
        }

        if (!error)
        {
                for (i=0; i<26; i++)
                {
                        if (strchr(line,i+'a') != NULL)
                        {
                                list_words_in(head.letter[i],line,
                                        mask);
                        }
                }
        }
}

void list_words_in(ENTRY_PTR ptr, char* letters,char* mask)
{
        char tword[MAXWORD];
        char rem[MAXLINE];

        if (ptr == NULL)
        {
                return;
        }

        if (in_word(letters,ptr->word,rem) && match (ptr->word,mask))
        {
                strcpy(tword,ptr->word);
                if (ptr->proper)
                {
                        *tword = toupper(*tword);
                }
                if (ptr->value > HIVAL)
                {
                        printf("<%s>\n",tword);
                }
                else
                {
                        printf("%s\n",tword);
                }
                fflush(stdout);
        }

        list_words_in(ptr->next,letters,mask);
}

int check_mask(char* mask)
{
        int i;
        static char vmask[] = "abcdefghijklmnopqrstuvwxyz+*";

        for (i=0; i < (int)strlen(mask); i++)
        {
                mask[i] = tolower(mask[i]);
                if (strchr(vmask,mask[i]) == NULL)
                {
                        printf("Invalid character (%c) in mask\n",mask[i]);
                        return TRUE;
                }

                if (mask[i] == '*' && mask[i+1] == '+')
                {
                        mask[i] = '+';
                        mask[i+1] = '*';
                }
        }
        return FALSE;
}

int match (char* word, char* mask)
{
        int i,j,k;
        char *ptr;

        i=j=0;
        while (mask[i] != '\0')
        {
                if (mask[i] == '*')
                {
                        i++;
                        if (mask[i] == '\0')
                        {
                                return TRUE;
                        }
                        for (k=i; mask[k] != '*' && mask[k] != '+' &&
                             mask[k] != '\0'; k++);
                        if (mask[k] == '\0')
                        {
                                if ((int) strlen(word+j) >= k-i)
                                {
                                        return (strcmp(word+j+strlen(word+j)-k+i,mask+i) == 0);
                                }
                                else
                                {
                                        return FALSE;
                                }
                        }
                        if ((ptr = subnstr(word+j,mask+i,k-i)) == NULL)
                        {
                                return FALSE;
                        }
                        j = ptr - word + k - i;
                        i = k;
                }
                else if (mask[i] == '+')
                {
                        i++;
                        if (word[j] == '\0')
                        {
                                return FALSE;
                        }
                        j++;
                }
                else
                {
                        if (mask[i] != word[j])
                        {
                                return FALSE;
                        }
                        i++;
                        j++;
                }
        }
        if (word[j] == '\0')
        {
                return TRUE;
        }

        return FALSE;
}

char *subnstr(char* str, char* sub, int n)
{
        int i;

        if (n == 0)
        {
                return str;
        }

        if (n == 1)
        {
                return strchr(str, *sub);
        }

        for (i=0; i <= (int)strlen(str)-n; i++)
        {
                if (strncmp(str+i,sub,n) == 0)
                {
                        return (str+i);
                }
        }

        return NULL;
}

void anag(char* letters, char* number)
{
        int i,m;
        char line[MAXLINE];
        char rem[MAXLINE];
        HEADER list;
        ENTRY_PTR ptr, nptr;

        if (*letters == '\0')
        {
                return;
        }
        if ((m = atoi(number)) == 0)
        {
                m = strlen(letters);
        }
        for (i=0; i<strlen(letters); i++)
        {
                letters[i] = tolower(letters[i]);
        }
        for (i=0; i<26; i++)
        {
                list.letter[i] = NULL;
                if (strchr(letters,'a'+i) != NULL)
                {
                        for (ptr = head.letter[i]; ptr != NULL;
                             ptr = ptr->next)
                        {
                                if (in_word(letters,ptr->word,rem))
                                {
                                        nptr = get_rec();
                                        strcpy(nptr->word,ptr->word);
                                        nptr->proper=ptr->proper;
                                        nptr->value=ptr->value;
                                        list.letter[i] = 
                                            add_entry(list.letter[i], nptr);
                                }
                        }
                }
        }
        *line = '\0';
        mix(list,letters,line,rem,0,0,m);
}

void mix(HEADER list, char* letters, char* line, char* rem,
	 int t, int l, int m)
{
        int i;
        char tline[MAXLINE];
        char trem[MAXLINE];
        char tword[MAXWORD];
        HEADER tlist;
        ENTRY_PTR ptr;

        if (l==m)
        {
                return;
        }

        for (i=0; i<26; i++)
        {
                tlist.letter[i] = list.letter[i];
        }


        for (i=0; i<26; i++)
        {
                if (strchr(letters,'a'+i) != NULL)
                {
                        for (ptr = list.letter[i]; ptr != NULL;
                             ptr = ptr->next)
                        {
                                if (in_word(letters,ptr->word,rem))
                                {
                                        strcpy(tline,line);
                                        strcpy(tword,ptr->word);
                                        if (ptr->proper)
                                        {
                                                *tword = toupper(*tword);
                                        }
                                        if (ptr->value > HIVAL)
                                        {
                                                strcat(tline,"<");
                                        }
                                        strcat(tline,tword);
                                        if (ptr->value > HIVAL)
                                        {
                                                strcat(tline,">");
                                        }
                                        strcat(tline," ");
                                        if (*rem == '\0')
                                        {
                                                printf("%-4d %s\n",t+ptr->value,tline);
                                                fflush(stdout);
                                        }
                                        else
                                        {
                                                strcpy(trem,rem);
                                                tlist.letter[i] = ptr;
                                                mix(tlist,trem,tline,
                                                             rem,t+ptr->value,
                                                             l+1,m);
                                        }
                                }
                        }
                }
                tlist.letter[i] = NULL;
        }
}

int in_word(char* letters, char* word, char* rem)
{
        int i,j,l,lt;
        char *s, temp[MAXLINE];

        if ((l = strlen(word)) > (lt = strlen(letters)))
        {
                return FALSE;
        }

        strcpy(temp, letters);
        for (i=0; i<l; i++)
        {
                if ((s = strchr(temp,word[i])) == NULL)
                {
                        return FALSE;
                }
                *s = ' ';
        }

        for (i=j=0; i<lt; i++)
        {
                if (temp[i] != ' ')
                {
                        rem[j++] = temp[i];
                }
        }
        rem[j] = '\0';
        return TRUE;
}

ENTRY_PTR add_entry(ENTRY_PTR list, ENTRY_PTR element)
{
        int r;
        
        if (list == NULL)
        {
                return element;
        }

        r = strcmp(list->word,element->word);

        if (r == 0)
        {
                free(element);
                return list;
        }

        if (r>0)
        {
                element->next = list;
                return element;
        }

        list->next = add_entry(list->next,element);
        return list;
}

