/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drparker@freenet.co.uk                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "xword.h"
#include "colour.h"
#include "font.h"
#include "image.h"
#include <iostream>
#include <sstream>
#include <unistd.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

using namespace std;

const std::string XWord::KEYS("abcdefghijklmnopqrstuvwxyz<+*");

void XWord::Initialise(Screen& s)
{
    bg = new Image(s, "2xgrep.bmp");
    font = new Font("font");
    letter_x = letter_y = 0;
    mode = FIND;
    pipefd[0] = pipefd[1] = -1;
    pid = -1;
    scroll = 0;
    max_words = 1;
    Help();
}

void XWord::Help()
{
    scroll = 0;
    output.clear();

    output.insert(output.end(), "2XWord");
    output.insert(output.end(), "");
    output.insert(output.end(), "MODES");
    output.insert(output.end(), "Find word including wildcards");
    output.insert(output.end(), "Find words in letter sequence");
    output.insert(output.end(), "Anagrams");
    output.insert(output.end(), "");
    output.insert(output.end(), "CONTROLS");
    output.insert(output.end(), "Y      Show this help");
    output.insert(output.end(), "A      Select letter");
    output.insert(output.end(), "B      Run Search");
    output.insert(output.end(), "X      Clear letters");
    output.insert(output.end(), "L/R    Scroll up/down");
    output.insert(output.end(), "Select Change Mode");
    output.insert(output.end(), "Start  Quit");
    output.insert(output.end(), "");
    output.insert(output.end(), "In anagram mode:");
    output.insert(output.end(), "Vol +/- Change max words");
}

void XWord::Render (Screen& s)
{
    const int VK_X = 250;
    const int VK_Y = 120;
    const Colour BG(s, 255, 255, 255);
    const Colour FG(s, 0, 0, 0);
    const Colour HIGHLIGHT(s, 255, 0, 0);

    int letter = letter_y * 5 + letter_x;
    bg->Put(s, 0,0);
    s.Rect(40, 5, 170, 15, BG);
    s.Rect(5, 30, 240, 190, BG);
    s.Rect(5, 225, 240, 15, BG);
    font->PutString(s, 45, 10, FG, Colour::NONE, letters); 
    font->PutString(s, VK_X, 190, BG, Colour::NONE, "< Delete");

    if (mode == FIND)
    {
	font->PutString(s, VK_X, 210, BG, Colour::NONE, "Wildcard");
	font->PutString(s, VK_X, 220, BG, Colour::NONE, "+ Single");
	font->PutString(s, VK_X, 230, BG, Colour::NONE, "* Any");
    }
    else if (mode == ANAGRAM)
    {
	string max_word_str;

	if (max_words > 0)
	{
	    ostringstream oss;
	    oss << "words: " << max_words;
	    max_word_str = oss.str();
	}

	font->PutString(s, VK_X, 210, BG, Colour::NONE, "Max");	
	font->PutString(s, VK_X, 220, BG, Colour::NONE, max_word_str);
    }

    font->PutString(s, 5, 230, FG, Colour::NONE, "Mode: ");

    for (int i=0; i < 19 && (scroll+i) < output.size(); i++)
    {
	font->PutString(s, 10, 30 + 10 * i, FG, Colour::NONE, 
			output[scroll+i]);
    }

    string mode_str;
    switch (mode)
    {
    case ANAGRAM:
	mode_str = "Anagrams";
	break;

    case FIND:
	mode_str = "Look up word";
	break;

    case SEQUENCE:
	mode_str = "Find words in sequence";
	break;

    }
	
    font->PutString(s, 50,230, FG, Colour::NONE, mode_str);
    
    int num_keys = 27;
    if (mode == FIND)
    {
	num_keys = 29;
    }
	
    for (int i=0; i<num_keys; i++)
    {
	Colour bc(Colour::NONE);
	if (i == letter)
	{
	    bc = HIGHLIGHT;
	}
	font->PutChar(s, (i%5)*10 + VK_X, (i/5)*10+ VK_Y, BG, bc, KEYS[i]); 	
    }
}

void XWord::ButtonChange(const Input& input, Input::ButtonType b)
{
    if (b == Input::START && input.IsPressed(b))
    {
	exit(0);
    }
    else if (b == Input::LEFT && input.IsPressed(b))
    {
	letter_x = (letter_x+4)%5;
    }
    else if (b == Input::RIGHT && input.IsPressed(b))
    {
	letter_x = (letter_x+1)%5;
    }
    else if (b == Input::DOWN && input.IsPressed(b))
    {
	letter_y = (letter_y+1)%6;
    }
    else if (b == Input::UP && input.IsPressed(b))
    {
	letter_y = (letter_y+5)%6;
    }
    else if (b == Input::A && input.IsPressed(b))
    {
	int letter = letter_y * 5 + letter_x;
	if (KEYS[letter] != '<')
	{
	    if (letters.length() < 20)
	    {
		letters += KEYS[letter];
	    }
	}
	else if (letters.length() > 0)
	{
	    letters.resize(letters.length()-1);
	}
    }
    else if (b == Input::SELECT && input.IsPressed(b))
    {
	if (mode == ANAGRAM) mode = FIND;
	else if (mode == FIND) mode = SEQUENCE;
	else if (mode == SEQUENCE) mode = ANAGRAM;
    }
    else if (b == Input::B && input.IsPressed(b))
    {
	Run();
    }
    else if (b == Input::L && input.IsPressed(b))
    {
	if (scroll > 0) scroll--;
    }
    else if (b == Input::R && input.IsPressed(b))
    {
	if ((scroll + 19) < output.size()) scroll++;
    }
    else if (b == Input::VOLUP && input.IsPressed(b))
    {
	max_words = (max_words % 9) + 1;
    }
    else if (b == Input::VOLDOWN && input.IsPressed(b))
    {
	max_words = ((max_words + 7) % 9) + 1;
    }
    else if (b == Input::X && input.IsPressed(b))
    {
	letters = "";
    }
    else if (b == Input::Y && input.IsPressed(b))
    {
	Help();
    }
}

void XWord::Run(void)
{
    if (pid != -1)
    {
	kill(pid, 9);
	pid = -1;
	int status;
	wait(&status);
    }

    string command;
    string option;
    switch (mode)
    {
    case ANAGRAM:
	command = "-a";
	if (max_words > 0)
	{
	    ostringstream oss;
	    oss << max_words;
	    option = oss.str();
	}

	break;

    case FIND:
	command = "-l";
	break;

    case SEQUENCE:
	command = "-s";
	break;

    }    

    // Reset output vector
    output.clear();
    output.insert(output.end(), "--- Start of list ---");
    scroll = 0;

    if (pipe(pipefd) < 0) perror("pipe");

    pid = fork();

    if (pid == 0)
    {
	// Child process - redirect stdout and stderr to pipe
	close(STDOUT_FILENO);
	dup2(pipefd[1], STDOUT_FILENO);
	dup2(STDOUT_FILENO, STDERR_FILENO);

	// Close read end of pipe
	close(pipefd[0]);

	// Exec xgrep
	if (option == "")
	{
	    execl("xgrep", "xgrep", command.c_str(), letters.c_str(), NULL);
	}
	else
	{
	    execl("xgrep", "xgrep", command.c_str(), letters.c_str(),
		  option.c_str(), NULL);
	}
 
	// Shouldn't get here
	perror("execl");
	return;
    }
    else
    {
	// Parent process - close write end of pipe
	close(pipefd[1]);
    }
}

void XWord::NonRender (const Input& input)
{
    char buf[1];
    int n;
    fd_set rfds, wfds, efds;

    // Check for output from xgrep
    if (pipefd[0] != -1)
    {
	FD_ZERO(&rfds);
	FD_ZERO(&wfds);
	FD_ZERO(&efds);
	FD_SET(pipefd[0], &rfds);
	timeval timeout;
	timeout.tv_sec = 0;
	timeout.tv_usec = 10000;

	n = select(pipefd[0]+1, &rfds, &wfds, &efds, &timeout);
	if (n == -1)
	{
	    perror("select");
	}
	else if (n > 0 && FD_ISSET(pipefd[0], &rfds))
	{
	    // Output on pipe
	    string line;
	    bool line_read = false;
	    while (!line_read && (n = read(pipefd[0], buf, 1)) > 0)
	    {
		if (buf[0] != '\n')
		{
		    line += buf[0];
		}
		else
		{
		    // Remove score - find first letter
		    string::size_type pos = 
			line.find_first_of("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");
		    if (pos != string::npos) line = line.substr(pos);
		    output.insert(output.end(), line);
		    line_read = true;
		}
	    }

	    if (n <= 0)
	    {
		// Pipe closed - end of output
		output.insert(output.end(), "--- End of list ---");
		close(pipefd[0]);
		pipefd[0] = -1;
		pid=-1;

		// Clear up child process
		int wait_status = 0;
		wait(&wait_status);
	    }
	}
    }


    // Do repeat keys
    if (input.Repeat(Input::L))
    {
	ButtonChange(input, Input::L);
    }
	
    if (input.Repeat(Input::R))
    {
	ButtonChange(input, Input::R);
    }
}
