/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drparker@freenet.co.uk                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef XWORD_H
#define XWORD_H
#include <vector>
#include <string>
#include "impl.h"
#include <time.h>

class Font;
class Image;

/**
 * Class XWord
 * 
 */
class XWord : public Impl 
{
public:

    // XWord modes
    enum Mode
    {
	ANAGRAM,
	FIND,
	SEQUENCE
    };

    // Initialisation
    virtual void Initialise (Screen& s);
    
    // Render screen
    virtual void Render (Screen& s);
    
    // Receive xgrep output
    virtual void NonRender (const Input& input);
    
    // Handle button presses
    virtual void ButtonChange(const Input& input, Input::ButtonType b);  
    

private:
    Font* font;               // Font
    Image* bg;                // Background image

    int letter_x;             // Keyboard x position
    int letter_y;             // Keyboard y position
    std::string letters;      // Search pattern
    static const std::string KEYS;   // Keyboard keys

    Mode mode;                // Current mode

    std::vector<std::string> output; // Output results 
    int pipefd[2];            // Pipe to/from xgrep
    pid_t pid;                // Process id of xgrep 
    int scroll;               // Output line at top of screen

    int max_words;            // Max words for anagrams

    void Run(void);           // Run xgrep
    void Help(void);          // Display help
};
#endif //XWORD_H

